@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "../regex.h"
 #include "../error.h"
@@ -561,6 +562,80 @@ builtInChr (int numArgs, macroArgument *args, environment *env, outputWriter *ow
     OUT_CHAR(ow, c);
 }
 
+void
+builtInSrange (int numArgs, macroArgument *args, environment *env, outputWriter *ow)
+{
+    int i;
+
+    if (!(numArgs % 2 == 0))
+    {
+	issueError(ERRMAC_WRONG_NUM_ARGS, "srange");
+	return;
+    }
+
+    for (i = 0; i < numArgs; i += 2)
+    {
+	int c1,
+	    c2;
+
+	transformArgumentToScalar(&args[i]);
+	transformArgumentToScalar(&args[i + 1]);
+
+	if (args[i].value.value->v.scalar.scalar.length != 1)
+	{
+	    issueError(ERRMAC_INVALID_MACRO_ARG, "srange",
+		       args[i].value.value->v.scalar.scalar.data);
+	    continue;
+	}
+	if (args[i + 1].value.value->v.scalar.scalar.length != 1)
+	{
+	    issueError(ERRMAC_INVALID_MACRO_ARG, "srange",
+		       args[i + 1].value.value->v.scalar.scalar.data);
+	    continue;
+	}
+	
+	c1 = args[i].value.value->v.scalar.scalar.data[0];
+	c2 = args[i + 1].value.value->v.scalar.scalar.data[0];
+
+	for (; c1 <= c2; ++c1)
+	    OUT_CHAR(ow, c1);
+    }
+}
+
+void
+builtInSmap (int numArgs, macroArgument *args, environment *env, outputWriter *ow)
+{
+    char map[256];
+    int i;
+
+    if (!(numArgs == 3))
+    {
+	issueError(ERRMAC_WRONG_NUM_ARGS, "smap");
+	return;
+    }
+
+    transformArgumentToScalar(&args[0]);
+    transformArgumentToScalar(&args[1]);
+    transformArgumentToScalar(&args[2]);
+
+    if (args[0].value.value->v.scalar.scalar.length !=
+	args[1].value.value->v.scalar.scalar.length)
+    {
+	issueError(ERRMAC_INVALID_MACRO_ARG, "smap",
+		   args[1].value.value->v.scalar.scalar.data);
+	return;
+    }
+
+    for (i = 0; i < 256; ++i)
+	map[i] = i;
+    for (i = 0; i < args[0].value.value->v.scalar.scalar.length; ++i)
+	map[args[0].value.value->v.scalar.scalar.data[i]] =
+	    args[1].value.value->v.scalar.scalar.data[i];
+
+    for (i = 0; i < args[2].value.value->v.scalar.scalar.length; ++i)
+	OUT_CHAR(ow, map[args[2].value.value->v.scalar.scalar.data[i]]);
+}
+
 void
 builtInNumber (int numArgs, macroArgument *args, environment *env, outputWriter *ow)
 {
@@ -687,6 +762,22 @@ builtInHexdecode (int numArgs, macroArgument *args, environment *env, outputWrit
     }
 }
 
+void
+builtInScrypt (int numArgs, macroArgument *args, environment *env, outputWriter *ow)
+{
+    char *result;
+
+    if (!(numArgs == 2))
+    {
+	issueError(ERRMAC_WRONG_NUM_ARGS, "scrypt");
+	return;
+    }
+
+    result = crypt(transformArgumentToScalar(&args[0])->v.scalar.scalar.data,
+		   transformArgumentToScalar(&args[1])->v.scalar.scalar.data);
+    OUT_STRING(ow, result, strlen(result));
+}
+
 void
 registerStringOps (void)
 {
@@ -704,9 +795,12 @@ registerStringOps (void)
     registerBuiltIn("ssub", builtInSubstring, 1, 0, 0);
     registerBuiltIn("scmp", builtInStrcmp, 1, 0, 0);
     registerBuiltIn("schr", builtInChr, 1, 0, 0);
+    registerBuiltIn("srange", builtInSrange, 1, 0, 0);
+    registerBuiltIn("smap", builtInSmap, 1, 0, 0);
     registerBuiltIn("snumber", builtInNumber, 1, 0, 0);
     registerBuiltIn("shexencode", builtInHexencode, 1, 0, 0);
     registerBuiltIn("shexdecode", builtInHexdecode, 1, 0, 0);
+    registerBuiltIn("scrypt", builtInScrypt, 1, 0, 0);
 
     re_syntax_options = CHPP_SYNTAX_BITS;
 }
