@@ -22,6 +22,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "config.h"
+
 #include <sys/param.h>
 #include <string.h>
 #include <stdio.h>
@@ -29,8 +31,13 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <assert.h>
-#ifdef _NEXT
-#include <libc.h>
+
+#ifndef PATH_MAX
+#ifdef MAXPATHLEN
+#define PATH_MAX MAXPATHLEN
+#else
+#define PATH_MAX 1024
+#endif
 #endif
 
 #include "memory.h"
@@ -81,7 +88,7 @@ main (int argc, char *argv[])
     registerInternals();
 
     defDirs = (dynstring*)memXAlloc(256 * sizeof(dynstring));
-    defDirs[0] = dsNewFrom("/usr/local/lib/chpp/include/");
+    defDirs[0] = dsNewFrom(CHPP_INCLUDEDIR "/");
     nrOfDefDirs = 1;
 
     while (1)
@@ -145,10 +152,11 @@ main (int argc, char *argv[])
 		    assert(length > 0);
 		    if (optarg[0] != '/')
 		    {
-			char currentPath[MAXPATHLEN + 1];
+			char currentPath[PATH_MAX];
 
-			getwd(currentPath);
+			getcwd(currentPath, PATH_MAX);
 			defDirs[nrOfDefDirs] = dsNewFrom(currentPath);
+			dsAppendChar(&defDirs[nrOfDefDirs], '/');
 		    }
 		    else
 			defDirs[nrOfDefDirs] = dsNew();
@@ -163,7 +171,7 @@ main (int argc, char *argv[])
 		break;
 		
 	    case 256 :
-		printf("chpp 0.3\n"
+		printf("chpp " VERSION "\n"
 		       "\n"
 		       "This program is free software; you can redistribute it and/or modify\n"
 		       "it under the terms of the GNU General Public License as published by\n"
@@ -244,7 +252,7 @@ main (int argc, char *argv[])
 			int result;
 			static char buffer[8192];
 
-			initCommands(inputFile, '#', argv[i]);
+			initCommands(inputFile, '#', argv[i], "./");
 			do
 			{
 			    result = fillBuffer(buffer, 8192);
@@ -256,9 +264,18 @@ main (int argc, char *argv[])
 		    {
 			bytecodeWriter *bcw = bcwNewOutput(globalEnvironment,
 							   &toplevelOutputWriter);
+			dynstring path = dsNewFrom(argv[i]),
+			    dir,
+			    file;
+
+			dsSplitPath(&path, &dir, &file);
+			if (dir.length == 0)
+			    dir = dsNewFrom("./");
+			else
+			    dsAppendChar(&dir, '/');
 
 			toplevelInputReader = irNewPreprocessor();
-			initCommands(inputFile, '#', argv[i]);
+			initCommands(inputFile, '#', file.data, dir.data);
 			currentFileName = dsNewFrom(argv[i]);
 			mainFileName = dsNewFrom(argv[i]);
 			addDependency(0, &mainFileName);
@@ -277,7 +294,7 @@ main (int argc, char *argv[])
 	    int result;
 	    static char buffer[8192];
 
-	    initCommands(stdin, '#', "-");
+	    initCommands(stdin, '#', "-", "./");
 	    do
 	    {
 		result = fillBuffer(buffer, 8192);
@@ -291,7 +308,7 @@ main (int argc, char *argv[])
 					       &toplevelOutputWriter);
 
 	    toplevelInputReader = irNewPreprocessor();
-	    initCommands(stdin, '#', "-");
+	    initCommands(stdin, '#', "-", "./");
 	    currentFileName = dsNewFrom("-");
 	    mainFileName = dsNewFrom("-");
 	    addDependency(0, &mainFileName);
